/// <reference path="../../typings/index.d.ts" />
"use strict";
var express = require('express');
var bodyParser = require('body-parser');
var logger = require('morgan');
var cookieParser = require('cookie-parser');
var http = require('http');
var Application = (function () {
    function Application() {
        this.Host = 'localhost';
        this.Port = 3000;
        Application.instance = this;
        this.initializeExpress();
        this.onBeforeApplicationStart();
    }
    // -- Initliaze
    Application.prototype.initializeExpress = function () {
        var app = express();
        app.use(logger('dev'));
        app.use(bodyParser.json());
        app.use(bodyParser.urlencoded({ extended: false }));
        app.use(cookieParser());
        app.get('/', function (req, res) {
            res.send('it worked');
        });
        this.express = app;
    };
    Application.prototype.initializeErrorHandler = function () {
        this.express.use('*', function (req, res, next) {
            res.status(404)
                .json({
                status: 404,
                message: 'Not Found',
                stacktrace: new Error().stack
            });
        });
        this.express.use(function (err, req, res, next) {
            if (err) {
                console.error(err.stack);
            }
            res.status(500)
                .json({
                status: 500,
                message: err.message,
                stacktrace: err.stack
            });
        });
    };
    // -- Hooks
    Application.prototype.onBeforeApplicationStart = function () {
    };
    // -- Register
    Application.prototype.registerGetRoutes = function (ResourceClass) {
        var _this = this;
        var getMethods = ResourceClass.constructor.__get;
        if (typeof getMethods !== 'undefined') {
            getMethods.forEach(function (getMethod) {
                _this.express.get(_this.getPath(ResourceClass, getMethod), function (req, res) {
                    console.log(getMethod);
                    var resource = new ResourceClass.constructor();
                    resource.req = req;
                    resource.res = res;
                    var paramList = _this.getParamList(ResourceClass, getMethod.methodName, req);
                    Promise.resolve(resource[getMethod.methodName].apply(resource, paramList)).then(function (returnValue) {
                        res.json(returnValue);
                    }).catch(function (err) { return res.status(err.statusCode || 500).json(err); });
                });
            });
        }
    };
    Application.prototype.registerPostRoutes = function (ResourceClass) {
        var _this = this;
        var postMethods = ResourceClass.__post;
        if (typeof postMethods !== 'undefined') {
            postMethods.forEach(function (postMethod) {
                _this.express.post(_this.getPath(ResourceClass, postMethod), function (req, res) {
                    var resource = new ResourceClass();
                    resource.req = req;
                    resource.res = res;
                    var paramList = _this.getParamList(ResourceClass, postMethod.methodName, req);
                    Promise.resolve(resource[postMethod.methodName].apply(resource, paramList)).then(function (returnValue) {
                        res.json(returnValue);
                    }).catch(function (err) { return res.status(err.statusCode || 500).json(err); });
                });
            });
        }
    };
    Application.prototype.registerPutRoutes = function (ResourceClass) {
        var _this = this;
        var putMethods = ResourceClass.__put;
        if (typeof putMethods !== 'undefined') {
            putMethods.forEach(function (putMethod) {
                _this.express.put(_this.getPath(ResourceClass, putMethod), function (req, res) {
                    var resource = new ResourceClass();
                    resource.req = req;
                    resource.res = res;
                    var paramList = _this.getParamList(ResourceClass, putMethod.methodName, req);
                    Promise.resolve(resource[putMethod.methodName].apply(resource, paramList)).then(function (returnValue) {
                        res.json(returnValue);
                    }).catch(function (err) { return res.status(err.statusCode || 500).json(err); });
                });
            });
        }
    };
    Application.prototype.registerDeleteRoutes = function (ResourceClass) {
        var _this = this;
        var deleteMethods = ResourceClass.__delete;
        if (typeof deleteMethods !== 'undefined') {
            deleteMethods.forEach(function (deleteMethod) {
                _this.express.delete(_this.getPath(ResourceClass, deleteMethod), function (req, res) {
                    var resource = new ResourceClass();
                    resource.req = req;
                    resource.res = res;
                    var paramList = _this.getParamList(ResourceClass, deleteMethod.methodName, req);
                    Promise.resolve(resource[deleteMethod.methodName].apply(resource, paramList)).then(function (returnValue) {
                        res.json(returnValue);
                    }).catch(function (err) { return res.status(err.statusCode || 500).json(err); });
                });
            });
        }
    };
    // -- Helpers
    Application.prototype.constructCorrectType = function (req, paramDef) {
        if (typeof paramDef.constructor !== 'undefined') {
            if (paramDef.isArray === true) {
                return req.body.map(function (item) {
                    return paramDef.constructor.fromJson(item);
                });
            }
            else {
                return paramDef.constructor.fromJson(req.body);
            }
        }
        else {
            return req.body;
        }
    };
    Application.prototype.getParamList = function (ResourceClass, methodName, req) {
        var _this = this;
        if (typeof ResourceClass.constructor.prototype[methodName].__params !== 'undefined') {
            return ResourceClass.constructor.prototype[methodName].__params.map(function (paramDef) {
                if (paramDef.paramType === 'PATH_PARAM') {
                    return req.params[paramDef.paramName];
                }
                else if (paramDef.paramType === 'QUERY_PARAM') {
                    return req.query[paramDef.paramName];
                }
                else if (paramDef.paramType === 'BODY_PARAM') {
                    return _this.constructCorrectType(req, paramDef);
                }
            });
        }
        else {
            return [];
        }
    };
    Application.prototype.getPath = function (ResourceClass, getMethod) {
        var path = [];
        if (this.Prefix != null) {
            path.push(this.Prefix);
        }
        path.push(ResourceClass.constructor.__path);
        var methodPath = ResourceClass.constructor.prototype[getMethod.methodName].__path;
        if (typeof methodPath !== 'undefined') {
            path.push(methodPath);
        }
        return path.join('');
    };
    // -- Public
    Application.prototype.registerResource = function (ResourceClass) {
        this.registerGetRoutes(ResourceClass);
        this.registerPostRoutes(ResourceClass);
        this.registerPutRoutes(ResourceClass);
        this.registerDeleteRoutes(ResourceClass);
    };
    Application.prototype.start = function () {
        var server = http.createServer(this.express);
        server.listen(this.Port, this.Host, function () {
            console.log('server listening at ', server.address().address + ':' + server.address().port);
        });
    };
    return Application;
}());
exports.Application = Application;
//# sourceMappingURL=application.js.map